#pragma once
#include <iostream>
#include <iomanip>
#include <vector>
#include <set>

struct Node 
{
    int id;
    double x, y ,z;
    bool vertex;

    Node() = default;
    Node(int, double, double, double, bool);

    bool operator==(const Node&) const;
    bool operator<(const Node&) const;
    friend std::ostream& operator<<(std::ostream&, const Node&);
    friend std::ostream& operator<<(std::ostream&, const std::vector<Node>&);
};

struct FiniteElement 
{
    int element_id;
    int material_id;
    std::vector<int> nodes_id;

    bool operator < (const FiniteElement& elem) const {
        return (element_id < elem.element_id);
    }
    friend std::ostream& operator<<(std::ostream&, const FiniteElement&);
    friend std::ostream& operator<<(std::ostream&, const std::vector<FiniteElement>&);
};

struct BoundaryFiniteElement 
{
    int element_id;
    int border_id;
	std::vector<int> nodes_id;
    friend std::ostream& operator<<(std::ostream&, const BoundaryFiniteElement&);
    friend std::ostream& operator<<(std::ostream&, const std::vector<BoundaryFiniteElement>&);
};

struct Edge 
{
	int head_id;
	int last_id;
	int self_id;
	Edge() = default;
	Edge(int, int);

	bool operator == (const Edge&) const;
};


class EdgeHash 
{
  public:
      std::size_t operator() (const Edge& _p) const;
};

std::ostream& operator << (std::ostream& out, const std::set <int>& elem);
std::ostream& operator << (std::ostream& out, const std::vector <int>& elem);





