#include "MeshLoader.h"

#include <iomanip>
#include <algorithm>
#include <list>
#include <set>
#include <unordered_set>

std::vector<Node>& MeshLoader::Get_Nodes() 
{
    return nodes;
}

std::vector<FiniteElement>& MeshLoader::Get_Elements() 
{
    return elements;
}

std::vector<BoundaryFiniteElement>& MeshLoader::Get_Surfaces() 
{
    return boundary;
}

void MeshLoader::Print_Data() 
{
    std::cout << Get_Nodes() << std::endl
              << Get_Elements() << std::endl
              << Get_Surfaces() << std::endl;
}


std::vector<int> MeshLoader::Get_Elements_by_ID(int id1, int id2, int id3) 
{ 
    std::vector<int> answer;
    auto current_iter = elements.begin();
    while (current_iter != elements.end()) 
	{
        current_iter = std::find_if(current_iter, elements.end(), [id1, id2, id3](const FiniteElement& current_element){
            return std::find(current_element.nodes_id.begin(), current_element.nodes_id.end(), id1) != current_element.nodes_id.end()
                && std::find(current_element.nodes_id.begin(), current_element.nodes_id.end(), id2) != current_element.nodes_id.end()
                && std::find(current_element.nodes_id.begin(), current_element.nodes_id.end(), id3) != current_element.nodes_id.end(); });
        if (current_iter != elements.end()) 
		{
            answer.push_back(current_iter->element_id);
            ++current_iter;
        }
    }
    return answer;
}

std::vector<int> MeshLoader::Get_Elements_by_rib(int id1, int id2) 
{
    std::vector<int> answer;

    auto current_iter = elements.begin();
    while (current_iter != elements.end()) 
	{
        current_iter = std::find_if(current_iter, elements.end(), [id1, id2](const FiniteElement& current_element){
            return std::find(current_element.nodes_id.begin(), current_element.nodes_id.end(), id1) != current_element.nodes_id.end()
                && std::find(current_element.nodes_id.begin(), current_element.nodes_id.end(), id2) != current_element.nodes_id.end(); });
        if (current_iter != elements.end()) 
		{
            answer.push_back(current_iter->element_id);
            ++current_iter;
        }
    }
    return answer;
}

std::set <int> MeshLoader::Get_Vertex_Nodes_by_Border_ID(int s_id) 
{
    std::set<int> answer;
    auto current_iter = boundary.begin();
    
    while (current_iter != boundary.end()) 
	{
        current_iter = std::find_if(current_iter, boundary.end(), [s_id](const BoundaryFiniteElement& s){ return s.border_id == s_id; });
        if (current_iter != boundary.end()) 
		{
            for (const auto& elem : current_iter->nodes_id) 
			{
				answer.insert(elem); 
            }
            ++current_iter;
        }
    }
	return answer;
}

std::set<int> MeshLoader::Get_Elements_by_ID_area(int a_id) 
{
	std::set<int> answer;
	auto current_iter = elements.begin();
	while (current_iter != elements.end())
	{
		current_iter = std::find_if(current_iter, elements.end(), [a_id](const FiniteElement& s) { return s.material_id == a_id; });
		if (current_iter != elements.end()) 
		{
			answer.insert(current_iter->element_id); 
			++current_iter;
		}
	}
	return answer;
}

std::vector<int> MeshLoader::Get_Borders_by_Border_ID(int s_id) 
{
    std::vector<int> answer;
    auto surface_by_ID = boundary.begin();
    while (surface_by_ID != boundary.end()) {
        surface_by_ID = std::find_if(surface_by_ID, boundary.end(), [s_id](const BoundaryFiniteElement& s){ 
			return s.border_id == s_id; });
        if (surface_by_ID != boundary.end()) 
		{
			answer.push_back(surface_by_ID->element_id);
            ++surface_by_ID;
        }
    }
    return answer;
}

std::vector<std::set<int>>  MeshLoader::Get_Nodes_Neighbors() 
{
	std::vector<std::set<int>> result(nodes.size()); 
	for (const auto& it_FiniteElement : elements)
	{
		for (auto& elem : it_FiniteElement.nodes_id)
		{
			result[elem - 1].insert(it_FiniteElement.nodes_id.begin(), it_FiniteElement.nodes_id.end());
			result[elem - 1].erase(elem);	
		}
	}
	
	return result;
}

void MeshLoader::Insert_New_Nodes() 
{
	std::unordered_set<Edge, EdgeHash> edge_container;
	for (auto& elem : elements)
	{
		std::vector<int> current_iter(elem.nodes_id);
		for (auto i = current_iter.begin(); i != current_iter.end(); i++) 
		{
			for (auto j = i + 1; j != current_iter.end(); j++)
			{
				Edge current_edge(*i, *j);
				auto search_edge = edge_container.find(current_edge);
				
				if (search_edge == edge_container.end())
				{
				   Node left_node = Get_Node_by_ID(*i); 
				   Node right_node = Get_Node_by_ID(*j); 
				   Node center_node(nodes.size() + 1, (left_node.x + right_node.x) / 2,
					(left_node.y + right_node.y) / 2, (left_node.z + right_node.z) / 2, false);
					current_edge.self_id = nodes.size() + 1;
					edge_container.insert(current_edge);
					nodes.push_back(center_node);
					elem.nodes_id.push_back(current_edge.self_id);
				}
				else
				{
					elem.nodes_id.push_back(search_edge->self_id);
				}
				
			}
		}
	}
	
	
	
	for (auto& it : boundary)
	{
		std::vector<int> current_iter(it.nodes_id);
		for (auto i = current_iter.begin(); i != current_iter.end(); i++) 
		{
			for (auto j = i + 1; j != current_iter.end(); j++)
			{
				Edge search_edge(*i, *j);
				auto search_point = edge_container.find(search_edge);
				if (search_point != edge_container.end())
				{
					it.nodes_id.push_back(search_point->self_id);
				}
			}
		}
	}
	
	
}


Node MeshLoader::Get_Node_by_ID(int n_id) 
{
	return nodes.at(n_id);
}