#pragma once
#include <fstream>
#include <vector>
#include <set>

#include "DataStructures.h"

class MeshLoader 
{
protected:
    std::vector<Node> nodes;
    std::vector<FiniteElement> elements;
    std::vector<BoundaryFiniteElement> boundary;
public:
    virtual void LoadMesh(const std::string&) = 0;
    virtual ~MeshLoader() = default;

    void Print_Data();

    // методы, позволяющие получить контейнеры узлов, конечных элементов и поверхностных элементов
    std::vector<Node>& Get_Nodes();
    std::vector<FiniteElement>& Get_Elements();
    std::vector<BoundaryFiniteElement>& Get_Surfaces();

    // метод, позволяющий найти КЭ по ID трех его вершинных узлов
    std::vector<int> Get_Elements_by_ID(int, int, int);

    // метод, позволяющий найти КЭ по ребру
    std::vector<int> Get_Elements_by_rib(int, int);

	// метод, возвращающий контейнер КЭ с заданным ID области 
	std::set<int> Get_Elements_by_ID_area(int);

    // метод, возвращающий контейнер граничных узлов по ID границы
    std::set <int> Get_Vertex_Nodes_by_Border_ID(int);

    // метод, возвращающий контейнер граниных КЭ с заданным ID границы
    std::vector<int> Get_Borders_by_Border_ID(int);

    // метод, производящий вставку новых узлов в середины ребер существующих КЭ
    void Insert_New_Nodes();

    // метод, создающий контейнер, n-ый элемент которого хранит контейнер всех соседей для узла n
	std::vector<std::set<int>> Get_Nodes_Neighbors();

    Node Get_Node_by_ID(int);
};

